'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});

var _createClass = function () { function defineProperties(target, props) { for (var i = 0; i < props.length; i++) { var descriptor = props[i]; descriptor.enumerable = descriptor.enumerable || false; descriptor.configurable = true; if ("value" in descriptor) descriptor.writable = true; Object.defineProperty(target, descriptor.key, descriptor); } } return function (Constructor, protoProps, staticProps) { if (protoProps) defineProperties(Constructor.prototype, protoProps); if (staticProps) defineProperties(Constructor, staticProps); return Constructor; }; }();

var _throttle = require('lodash/throttle');

var _throttle2 = _interopRequireDefault(_throttle);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function _classCallCheck(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }

var Chart = function () {
  function Chart() {
    _classCallCheck(this, Chart);

    this.options = {
      innerRadius: 0,
      strokeWidth: 2,
      markerSize: 0,
      sorted: false,
      orientation: 'v',
      baselineValue: 0,
      hideXaxis: false,
      hideYaxis: false,
      showGridLines: false,
      showXgridlines: false,
      showYgridlines: false,
      guideLine: 'none',
      legendPosition: 'r',
      animated: true
    };
    this.resizeHandler = (0, _throttle2.default)(this.resizeHandler, 200).bind(this);
    this._onMount = [];
    this._onUnmount = [];
    this._onUpdate = [];
    this._onResize = [];
  }

  _createClass(Chart, [{
    key: 'update',
    value: function update(nextProps) {
      this.datasets = nextProps.data.map(function (s) {
        return new Plottable.Dataset(s.series, s.label);
      });
      this.plot.datasets(this.datasets);
      this.options.data = nextProps.data;
      this.onUpdate(nextProps);
    }
  }, {
    key: '_setGridlines',
    value: function _setGridlines(props, scale) {
      if (props.showGridlines) {
        var horizontal = props.orientation === 'h';
        var xScale = horizontal ? scale : null;
        var yScale = horizontal ? null : scale;
        this.gridlines = new Plottable.Components.Gridlines(xScale, yScale);
      }
    }
  }, {
    key: '_setAxes',
    value: function _setAxes(props, scale, categoryScale) {
      var horizontal = props.orientation === 'h';
      if (!props.hideXaxis) {
        if (horizontal) {
          this.xAxis = new Plottable.Axes.Numeric(scale, 'bottom');
        } else {
          this.xAxis = categoryScale instanceof Plottable.Scales.Time ? new Plottable.Axes.Time(categoryScale, 'bottom') : new Plottable.Axes.Category(categoryScale, 'bottom');
        }
        this.layout.add(this.xAxis, 1, 2);
      }
      if (!props.hideYaxis) {
        if (horizontal) {
          this.yAxis = categoryScale instanceof Plottable.Scales.Time ? new Plottable.Axes.Time(categoryScale, 'left') : new Plottable.Axes.Category(categoryScale, 'left');
        } else {
          this.yAxis = new Plottable.Axes.Numeric(scale, 'left');
        }
        this.layout.add(this.yAxis, 0, 1);
      }
      if (props.xLabel) {
        this.layout.add(new Plottable.Components.AxisLabel(props.xLabel), 2, 2);
      }
      if (props.yLabel) {
        this.layout.add(new Plottable.Components.AxisLabel(props.yLabel, -90), 0, 0);
      }
    }
  }, {
    key: '_setLegend',
    value: function _setLegend(props, colorScale) {
      this.legend = new Plottable.Components.Legend(colorScale).xAlignment('center').yAlignment('center');
      switch (props.legendPosition) {
        case 't':
          this.layout = new Plottable.Components.Table([[this.legend.maxEntriesPerRow(Infinity)], [this.layout]]).rowPadding(10);
          break;
        case 'r':
          this.layout = new Plottable.Components.Table([[this.layout, this.legend]]).columnPadding(10);
          break;
        case 'b':
          this.layout = new Plottable.Components.Table([[this.layout], [this.legend.maxEntriesPerRow(Infinity)]]).rowPadding(10);
          break;
        case 'l':
          this.layout = new Plottable.Components.Table([[this.legend, this.layout]]).columnPadding(10);
          break;
      }
    }
  }, {
    key: '_setInteractions',
    value: function _setInteractions(props) {
      var _this = this;

      if (props.clickHandler) {
        new Plottable.Interactions.Click().onClick(function (point) {
          var target = _this.plot.entitiesAt(point)[0];
          props.clickHandler(target, _this.plot.entities());
        }).attachTo(this.plot);
      }

      if (props.hoverHandler) {
        new Plottable.Interactions.Pointer().onPointerMove(function (point) {
          var target = _this.plot.entitiesAt(point)[0];
          props.hoverHandler(target, _this.plot.entities());
        }).onPointerExit(function (point) {
          props.hoverHandler(null, _this.plot.entities());
        }).attachTo(this.plot);
      }
    }
  }, {
    key: 'resizeHandler',
    value: function resizeHandler() {
      this.layout.redraw();
      this.onResize();
    }
  }, {
    key: 'mount',
    value: function mount(element) {
      this.layout.renderTo(element);
      window.addEventListener('resize', this.resizeHandler);
      this.onMount(element);
    }
  }, {
    key: 'unmount',
    value: function unmount() {
      this.onUnmount();
      window.removeEventListener('resize', this.resizeHandler);
      this.layout.detach();
    }
  }, {
    key: 'onMount',
    get: function get() {
      return function (element) {
        var _this2 = this;

        this._onMount.forEach(function (cb) {
          cb.call(_this2, element);
        });
      };
    },
    set: function set(cb) {
      this._onMount.push(cb);
    }
  }, {
    key: 'onUnmount',
    get: function get() {
      return function (element) {
        var _this3 = this;

        this._onUnmount.forEach(function (cb) {
          cb.call(_this3);
        });
      };
    },
    set: function set(cb) {
      this._onUnmount.push(cb);
    }
  }, {
    key: 'onUpdate',
    get: function get() {
      return function (nextProps) {
        var _this4 = this;

        Plottable.Utils.DOM.requestAnimationFramePolyfill(function () {
          _this4._onUpdate.forEach(function (cb) {
            cb.call(_this4, nextProps);
          });
        });
      };
    },
    set: function set(cb) {
      this._onUpdate.push(cb);
    }
  }, {
    key: 'onResize',
    get: function get() {
      return function () {
        var _this5 = this;

        Plottable.Utils.DOM.requestAnimationFramePolyfill(function () {
          _this5._onResize.forEach(function (cb) {
            cb.call(_this5);
          });
        });
      };
    },
    set: function set(cb) {
      this._onResize.push(cb);
    }
  }]);

  return Chart;
}();

exports.default = Chart;