'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});

var _Chart2 = require('./Chart');

var _Chart3 = _interopRequireDefault(_Chart2);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function _classCallCheck(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }

function _possibleConstructorReturn(self, call) { if (!self) { throw new ReferenceError("this hasn't been initialised - super() hasn't been called"); } return call && (typeof call === "object" || typeof call === "function") ? call : self; }

function _inherits(subClass, superClass) { if (typeof superClass !== "function" && superClass !== null) { throw new TypeError("Super expression must either be null or a function, not " + typeof superClass); } subClass.prototype = Object.create(superClass && superClass.prototype, { constructor: { value: subClass, enumerable: false, writable: true, configurable: true } }); if (superClass) Object.setPrototypeOf ? Object.setPrototypeOf(subClass, superClass) : subClass.__proto__ = superClass; }

/**
 * @typedef {Object} StackedBar
 * @property {Object} layout
 * @property {Object} plot
 * @property {Object} legend
 * @property {Object} xAxis
 * @property {Object} yAxis
 * @property {Object} xLabel
 * @property {Object} yLabel
 * @property {Object} gridlines
 * @property {Function} mount
 * @property {Function} update
 * @property {Function} unmount
 * @property {Object} options
 */

var StackedBar = function (_Chart) {
  _inherits(StackedBar, _Chart);

  /**
   * @param {Object} props
   * @param {string[]} props.labels - required
   * @param {Object[]} props.traces - required
   * @param {string[]} props.traces.labels - required
   * @param {number[]} props.traces.values - required
   * @param {Object} props.scale - default new Plottable.Scales.Linear()
   * @param {Object} props.categoryScale - default new Plottable.Scales.Category()
   * @param {Object} props.colorScale - default new Plottable.Scales.Color()
   * @param {('h'|'v')} props.orientation - default 'h'
   * @param {number} props.baselineValue - default 0
   * @param {Function} props.labelFormatter - optional
   * @param {boolean} props.showGridlines - default false
   * @param {boolean} props.hideXaxis - default false
   * @param {boolean} props.hideYaxis - default false
   * @param {string} props.xLabel - optional
   * @param {string} props.yLabel - optional
   * @param {('t'|'r'|'b'|'l'|'none')} props.legendPosition - default 'r'
   * @param {boolean} props.animated - default true
   * @param {Function} props.clickHandler - optional
   * @param {Function} props.hoverHandler - optional
   *
   * @return {StackedBar}
   */
  function StackedBar(props) {
    _classCallCheck(this, StackedBar);

    var _this = _possibleConstructorReturn(this, (StackedBar.__proto__ || Object.getPrototypeOf(StackedBar)).call(this));

    props = Object.assign(_this.options, props);

    var scale = props.scale || new Plottable.Scales.Linear();
    var categoryScale = props.categoryScale || new Plottable.Scales.Category();
    var colorScale = props.colorScale || new Plottable.Scales.Color();

    var horizontal = props.orientation === 'h';
    var plotType = horizontal ? Plottable.Plots.Bar.ORIENTATION_HORIZONTAL : Plottable.Plots.Bar.ORIENTATION_VERTICAL;
    _this.plot = new Plottable.Plots.StackedBar(plotType).attr('fill', function (d, i, dataset) {
      return dataset.metadata();
    }, colorScale).labelsEnabled(false).animated(props.animated).baselineValue(props.baselineValue);
    _this.plot[horizontal ? 'x' : 'y'](function (d) {
      return d.value;
    }, scale);
    _this.plot[horizontal ? 'y' : 'x'](function (d) {
      return d.label;
    }, categoryScale);

    if (props.data) {
      _this.datasets = props.data.map(function (s) {
        return new Plottable.Dataset(s.series, s.label);
      });
      _this.datasets.forEach(function (dataset) {
        _this.plot.addDataset(dataset);
      });
    }

    if (props.labelFormatter) {
      _this.plot.labelFormatter(_this.props.labelFormatter).labelsEnabled(true);
    }

    _this._setGridlines(props, scale);
    var plotArea = _this.gridlines ? new Plottable.Components.Group([_this.gridlines, _this.plot]) : _this.plot;

    _this.layout = new Plottable.Components.Table([[null, null, plotArea], [null, null, null], [null, null, null]]);

    _this._setAxes(props, scale, categoryScale);
    _this._setLegend(props, colorScale);
    _this._setInteractions(props);
    return _this;
  }

  return StackedBar;
}(_Chart3.default);

exports.default = StackedBar;